unit CREEQ;

interface

uses Windows, SysUtils, Forms;

function GetCRC32(FileName: string): string;
procedure BuildCRCTable;
function RecountCRC(b: byte; CrcOld: Longint): Longint;
function HextW(w: Word): string;
function HextL(l: Longint): string;
function GetDiff(File1,File2,File3:String):Cardinal;
function CountDiffs(Diffs:String):Integer;
function DateiSize(FileName:String):Cardinal;

var
  CRCTable: array[0..512] Of Longint;

const
  CRCPOLY = $EDB88320;

type
  Long = record
    LoWord: Word;
    HiWord: Word;
  end;

implementation

function DateiSize(FileName:String):Cardinal;
var f: File of Byte;
begin
  if FileExists(FileName) then
  begin
    AssignFile(f, FileName);
    Reset(f);
    Result := FileSize(f);
    CloseFile(f);
  end else Result := 0;
end;

function CountDiffs(Diffs:String):Integer;
var i,a: Integer;
begin
  a := 0;
  for i := 1 to Length(Diffs) do
    if Diffs[i] = ',' then Inc(a);
  Result := a DIV 2;
end;

function GetDiff(File1,File2,File3:String):Cardinal;
var F1, F2: File;
    F3: TextFile;
    Pos, Nr: Cardinal;
    In1, In2: Char;
begin
  Pos := 0;
  Nr := 0;
  FileMode := 2;
  AssignFile(F1, File1);
  AssignFile(F2, File2);
  AssignFile(F3, File3);
  Reset(F1,1);
  Reset(F2,1);
  ReWrite(F3);
  while not (FilePos(F2) = FileSize(F2)) do
  begin
    In1 := ' ';
    In2 := ' ';
    if not (FilePos(F1) = FileSize(F1)) then
    begin
      Seek(F1, Pos);
      BlockRead(F1, In1, 1);
    end;
    if not (FilePos(F2) = FileSize(F2)) then
    begin
      Seek(F2, Pos);
      BlockRead(F2, In2, 1);
    end;
    if not (In1 = In2) then
    begin
      Write(F3, IntToHex(Pos, 8) + IntToHex(Ord(In2),2));
      Inc(Nr);
    end;
    Inc(Pos);
  end;
  CloseFile(F1);
  CloseFile(F2);
  CloseFile(F3);
  Result := Nr;
end;

function HextW(w: Word): string;
const
  h: array[0..15] Of char = '0123456789ABCDEF';
begin
  HextW := '';
  HextW := h[Hi(w) shr 4] + h[Hi(w) and $F] + h[Lo(w) shr 4]+h[Lo(w) and $F];
end;

function HextL(l: Longint): string;
begin
  with Long(l) do
    HextL := HextW(HiWord) + HextW(LoWord);
end;

function RecountCRC(b: byte; CrcOld: Longint): Longint;
begin
  RecountCRC := CRCTable[byte(CrcOld xor Longint(b))] xor ((CrcOld shr 8) and $00FFFFFF)
end;

procedure BuildCRCTable;
var
  i, j: Word;
  r: Longint;
begin
  FillChar(CRCTable, SizeOf(CRCTable), 0);
  for i := 0 to 255 do
  begin
    r := i shl 1;
    for j := 8 downto 0 do
      if (r and 1) <> 0 then
        r := (r Shr 1) xor CRCPOLY
      else
        r := r shr 1;
    CRCTable[i] := r;
   end;
end;

function GetCRC32(FileName: string): string;
var
  Buffer: PChar;
  f: File of Byte;
  b: array[0..255] of Byte;
  CRC: Longint;
  e, i: Integer;
begin
  BuildCRCTable;
  CRC := $FFFFFFFF;
  AssignFile(F, FileName);
  FileMode := 0;
  Reset(F);
  GetMem(Buffer, SizeOf(B));
  repeat
    FillChar(b, SizeOf(b), 0);
    BlockRead(F, b, SizeOf(b), e);
    for i := 0 to (e-1) do
     CRC := RecountCRC(b[i], CRC);
  until (e < 255) or (IOresult <> 0);
  FreeMem(Buffer, SizeOf(B));
  CloseFile(F);
  CRC := Not CRC;
  Result := '$' + HextL(CRC);
end;

end.
